// Copyright 2019 Finobo
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//      http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package stores

import (
	"testing"

	"github.com/pkg/errors"
	ldberr "github.com/syndtr/goleveldb/leveldb/errors"
)

func TestIsNotFoundError(t *testing.T) {
	type args struct {
		err error
	}
	tests := []struct {
		name string
		args args
		want bool
	}{
		{
			"ErrNotFound-not-found",
			args{
				errors.Errorf("not found"),
			},
			true,
		},
		{
			"ErrNotFound-leveldb-with-stack",
			args{
				errors.WithStack(ldberr.ErrNotFound),
			},
			true,
		},
		{
			"unknown error",
			args{
				errors.Errorf("unknown error"),
			},
			false,
		},
	}
	for _, tt := range tests {
		t.Run(tt.name, func(t *testing.T) {
			if got := IsNotFoundError(tt.args.err); got != tt.want {
				t.Errorf("IsNotFoundError() = %v, want %v", got, tt.want)
			}
		})
	}
}
